#include <iostream>
#include <cstdlib>
#include <cstdio>
#include <vector>
#include <cmath>
#include <algorithm>
#include "MersenneTwister.h"
using namespace std;

MTRand rand1;
const double maxSquareCoord = 100;

struct city
{
	double x, y;
};
unsigned long n_factorial(unsigned long n)
{
	unsigned long factorial = 1;
	for(unsigned long q = n; q > 0; q--)
		factorial*=q;
	return factorial;
}
unsigned long n_sum(unsigned long n)
{
	unsigned sum = 0;
	for(unsigned long q = n; q > 0; q--)
		sum+=q;
	return sum;
}
void initialize(city** Cities, unsigned long* numCities)
{
	*Cities = (city*)malloc(sizeof(city)*(*numCities));
	for(int u = 0; u < *numCities; u++)
	{
		(*Cities)[u].x = rand1.randInt(maxSquareCoord);
		(*Cities)[u].y = rand1.randInt(maxSquareCoord);
	}
}
void print(const int *v, const int size, int* permutations)
{
  if (v != 0) {
    for (int i = 0; i < size; i++) {
		static int r = 0;
		permutations[r] = v[i];
		r++;
		///printf("%4d", v[i] );
    }
   /// printf("\n");
  }
}
void swap(int *v, const int i, const int j)
{
  int t;
  t = v[i];
  v[i] = v[j];
  v[j] = t;
} 

void rotateLeft(int *v, const int start, const int n)
{
  int tmp = v[start];
  for (int i = start; i < n-1; i++) {
    v[i] = v[i+1];
  }
  v[n-1] = tmp;
} 

void permute(int *v, const int start, const int n, int* permutations)
{
  print(v, n, permutations);
  if (start < n) {
    int i, j;
    for (i = n-2; i >= start; i--) {
      for (j = i + 1; j < n; j++) {
	swap(v, i, j);
	permute(v, i+1, n, permutations);
      } // for j
      rotateLeft(v, i, n);
    } // for i
  }
}
void init(int *v, int N)
{
  for (int i = 0; i < N; i++) {
    v[i] = i+1;
  }
} 

double findDistance(city* Cities, int* permutation, int index, int numCities)
{
	//index = where to look in permutation array for city
	int citiesTraversed = 0;
	double distance = 0;
	int origIndex = index;
	while(citiesTraversed < numCities)
	{
		city* curCity = &Cities[ permutation[index] -1];
		city* nextCity;
		if((index+1)%numCities ==  0)
			nextCity = &Cities[ permutation[origIndex]-1 ];
		else
			nextCity = &Cities[ permutation[index+1]-1 ];

		distance+=sqrt( pow(double(curCity->x - nextCity->x), 2.0) + pow(double(curCity->y - nextCity->y), 2.0));
		index++;
		citiesTraversed++;
	}
	return distance;
}


void findDistances(city* Cities, unsigned long numCities)
{
	double* distances = (double*)malloc(sizeof(double)*n_factorial(numCities));	
	int *set = new int[numCities];
	int *permutations = (int*)malloc(sizeof(int)*n_factorial(numCities)*numCities);
	vector<double> paths;

	//////Generate Permutations of Travelling//////
    init(set, numCities);
    permute(set, 0, numCities, permutations);
	//cout << "The following paths are possible: " << endl;
	for(int q = 0; q < (n_factorial(numCities)); q++)
	{
		double temp = findDistance(Cities, permutations, q*numCities, numCities);
		paths.push_back(temp);
	//	cout << "PATH #" << q+1 << ": " << temp << endl;
	}
   
	vector<double>::iterator result = min_element(paths.begin(), paths.end());

	cout << "Shortest path is of length " << *result << endl;
	cout << "Shortest path is path # " << distance(paths.begin(), result) + 1 << ": ";
	for(int h = distance(paths.begin(), result)*numCities; h < distance(paths.begin(), result)*numCities + numCities; h++)
		cout <<  permutations[h] << " ";

	cout << endl;
	delete [] set;
	free(permutations);
}

int main()
{
	unsigned long numCities=10;
	city* Cities = 0;//(city*)malloc(sizeof(city)*numCities);
	initialize(&Cities, &numCities);

	findDistances(Cities, numCities);
	free(Cities);
	system("PAUSE");
	return 0;
}